package org.openxma.addons.ui.table.customizer.xma.server;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.openxma.addons.ui.table.customizer.common.ColumnMetaData;
import org.openxma.addons.ui.table.customizer.common.CommonColumnModel;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.CommonTableModel;
import org.openxma.addons.ui.table.customizer.common.CommonUserModel;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.FilterOperator;
import org.openxma.addons.ui.table.customizer.mdl.dto.UserTableView;
import org.openxma.addons.ui.table.customizer.mdl.service.TableCustomizerService;
import org.openxma.dsl.platform.query.Expression;
import org.openxma.dsl.platform.query.QueryObject;
import org.springframework.beans.factory.annotation.Autowired;

import at.spardat.xma.rpc.RemoteCall;
import at.spardat.xma.rpc.RemoteReply;
import at.spardat.xma.session.XMASessionServer;

/**
 * This class implents the server part of TableCustomizerComp.
 *
 */

public class TableCustomizerComp extends TableCustomizerCompGen implements ITableCustomizerServer {

    @Autowired
    private TableCustomizerService tableCustomizerService;

    private CommonUserModel userModel;

    private Map<String, ColumnMetaData> columnMap = null;

    public void setUserModel(CommonUserModel userModel) {
        this.userModel = userModel;
    }

    /**
     * Initializes the TableCustomizerComp. This method is called by the XMA-Runtime.
     *
     * @param session
     *            the XMASession, this TableCustomizerComp belongs to.
     * @param id
     *            the unique id of this TableCustomizerComp.
     */
    public TableCustomizerComp(XMASessionServer session, short id) {
        super(session, id);
    }

    public QueryObject getFilterQueryForTable() {
        return getQueryObject();
    }

    /**
     * initial call of client when page registers its customizer at server. <br>
     * if there is no tableModel on the server yet, the server is updated with a tableModel from the client.
     * <p>
     * 
     * @see org.openxma.addons.ui.table.customizer.xma.client.TableCustomizerComp#init()
     * @param call
     * @param reply
     */
    @SuppressWarnings("unchecked")
    public void init(RemoteCall call, RemoteReply reply) {
        this.columnMap = (Map<String, ColumnMetaData>) call.getParameter(0);
        CommonTableModel tableModel = getTableModel();
        if (tableModel == null) {
            tableModel = (CommonTableModel) call.getParameter(1);
            getUserModel().setTableModel(tableModel);
        }
        reply.setParameter(0, tableModel); // update UI on client
        
    }
    public void register(RemoteCall call, RemoteReply reply) {
        // dummy method to register component on server
    }

    public QueryObject getQueryObject(int sortingColumn, boolean isAscending) {
        return getQueryObjectInternal(sortingColumn, isAscending);
    }

    public QueryObject getQueryObject() {
        return getQueryObjectInternal(-1, false);
    }

    private String getColumnAttributeByIndex(int i) {
        for (Entry<String, ColumnMetaData> entry : columnMap.entrySet()) {
            if ((Integer) entry.getValue().getIndex() == i) {
                return entry.getKey();
            }
        }
        return null;
    }    

    private QueryObject getQueryObjectInternal(int sortingColumn, boolean isAscending) {
        QueryObject queryObject = new QueryObject();

        CommonTableModel tableModel = getTableModel();
        if (tableModel == null) {
            if (sortingColumn != -1) {
                String attr = getColumnAttributeByIndex(sortingColumn);
                if (attr != null) {
                    if (isAscending) {
                        queryObject = queryObject.orderBy(queryObject.get(attr).asc());
                    } else {
                        queryObject = queryObject.orderBy(queryObject.get(attr).desc());
                    }
                }
            }
            return queryObject;
        } else {
            List<CommonColumnModel> columnModels = tableModel.getColumnModelsSortedByNumSort();
            List<Expression> orderExpressions = new ArrayList<Expression>();
            Expression whereExpression = null;
            for (CommonColumnModel column : columnModels) {
                Expression exInner = null;
                Expression colExpr = queryObject.get(column.getNamColumn());
                if (column.getFilterModel() != null && column.getFilterModel().size() > 0) {
                    for (CommonFilterModel filter : column.getFilterModel()) {
                        FilterOperator fop = filter.getCodFilterOperator();
                        Object val = filter.getValue(column.getCodDataType(), column.getTxtFormatterPattern(), filter
                                .getTxtValue());
                        Expression exp = null;
                        switch (fop) {
                        case eq:
                            exp = colExpr.eq(val);
                            break;
                        case neq:
                            exp = colExpr.neq(val);
                            break;
                        case gt:
                            exp = colExpr.gt(val);
                            break;
                        case ge:
                            exp = colExpr.gte(val);
                            break;
                        case le:
                            exp = colExpr.lte(val);
                            break;
                        case like:
                            exp = colExpr.like(val);
                            break;
                        case lt:
                            exp = colExpr.lt(val);
                            break;
                        case notlike:
                            exp = colExpr.notLike(val);
                            break;
                        default:
                            break;
                        }
                        if (column.isNullableFilter()) {
                            exp = exp.or(colExpr.isNull());
                        }
                        if (exInner == null) {
                            exInner = exp;
                        } else {
                            if (Boolean.TRUE.equals(column.isAndFilter())) {
                                exInner = exInner.and(exp);
                            } else {
                                exInner = exInner.or(exp);
                            }
                        }
                    }
                } // end for column
                if (exInner != null) {
                    if (null == whereExpression) {
                        whereExpression = exInner;
                    } else {
                        whereExpression = whereExpression.and(exInner);
                    }
                }
                if (sortingColumn == -1) {
                    if (column.isToBeSorted() && column.getSortOrder() >= 0) {
                        if (column.isAscending()) {
                            orderExpressions.add(colExpr.asc());

                        } else {
                            orderExpressions.add(colExpr.desc());
                        }
                    }
                } else {
                    if (column.getNumColumnIndex().intValue() == sortingColumn) {
                        if (isAscending) {
                            orderExpressions.add(colExpr.asc());
                        } else {
                            orderExpressions.add(colExpr.desc());
                        }
                    }
                }
            }
            QueryObject where = queryObject.where(whereExpression);
            for (Expression orderBy : orderExpressions) {
                where = where.orderBy(orderBy);
            }
            return where;
        }
    }

    private CommonTableModel getPersistedTableModel() {
        return tableCustomizerService.getTableModel(getUser(), getMandant(), getFQTableName());
    }

    protected CommonUserModel getUserModel() {
        if (userModel == null) {
            int numMandant = getMandant();
            String name = getUser();
            CommonUserModel userModelFromRepository = tableCustomizerService.getUserModel(name, numMandant);
            if (userModelFromRepository == null) {
                userModel = new CommonUserModel();
                userModel.setCodUser(name);
                userModel.setNumMandant(numMandant);
            } else {
                userModel = userModelFromRepository;
            }
        }
        return userModel;
    }

    private String getUser() {
        return getSession().getContext().getName();
    }

    private int getMandant() {
        String mandant = getSession().getContext().getMandant();
        if (mandant == null || mandant.trim().equals("")) {
            return 0;
        } else {
            return Integer.parseInt(mandant);
        }
    }

    protected String getFQTableName() {
        return getFQTableNameProperty();
    }

    protected UserTableView getUserTableView(String user, int mandant) {
        return tableCustomizerService.findTableViewForUser(user, mandant, getFQTableName());
    }

    protected void persist(CommonTableModel tableModel) {
        tableCustomizerService.persist(tableModel, getUser(), getMandant());
    }
    public void removeAllFilter(RemoteCall call, RemoteReply reply) {
        CommonTableModel tableModel = getTableModel();
        if (tableModel!= null) {
            tableModel.removeAllFilter();            
        }
    }

    public void setFilter(RemoteCall call, RemoteReply reply) {

        CommonTableModel uiTableModel = ((CommonTableModel) call.getParameter(0));
        int columnId = ((Integer) call.getParameter(1));
        boolean isAnd = ((Boolean) call.getParameter(2));
        List<CommonFilterModel> filters = (List<CommonFilterModel>) call.getParameter(3);
        CommonTableModel sessionTableModel = getUserModel().getTableModel(uiTableModel.getNamTable());
        if (sessionTableModel != null) {
           
            CommonColumnModel columnModelByColIndex = sessionTableModel.getColumnModelByColIndex(columnId);
            columnModelByColIndex.setAndFilter(isAnd);
            columnModelByColIndex.setFilterModel(filters);
        } else {
           
            CommonColumnModel columnModelByColIndex = uiTableModel.getColumnModelByColIndex(columnId);
            columnModelByColIndex.setAndFilter(isAnd);
            columnModelByColIndex.setFilterModel(filters);
            userModel.setTableModel(uiTableModel);
        }
    }



    public void onGetTableModel(RemoteCall call, RemoteReply reply) {
        CommonTableModel tableModel = getTableModel();
        if (tableModel != null) {
            reply.setParameter(0, tableModel); // update UI on client
        }
    }

    public CommonTableModel getTableModel() {
        return getUserModel().getTableModel(getFQTableName());
    }
    
    public Map<String, ColumnMetaData> getColumnMap(){
        return columnMap;
    }

}
